# jinja-tools
#
# Various tools for Jinja2,
# including new filters and tests based on python-moretools,
# a JinjaLoader class for Django,
# and a simple JinjaBuilder class for SCons.
#
# Copyright (C) 2011-2013 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# jinja-tools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# jinja-tools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with jinja-tools.  If not, see <http://www.gnu.org/licenses/>.

"""jinjatools.scons

.. moduleauthor:: Stefan Zimmermann <zimmermann.code@gmail.com>
"""
__all__ = ['JinjaBuilder']

from SCons.Builder import BuilderBase
from SCons.Action import Action

import jinjatools


class JinjaBuilder(BuilderBase):
    def __init__(self, jinja_loader, context={}):
        BuilderBase.__init__(
          self, action=Action(self.jinja_action),
          src_suffix='.jinja', suffix='')

        self.jinja_env = jinjatools.Environment(loader=jinja_loader)
        self.jinja_template_context = context

    def jinja_action(self, target, source, env):
        context = dict(self.jinja_template_context)
        try:
            context.update(env['JINJACONTEXT'])
        except KeyError:
            pass

        options = {}
        try:
            options['loader'] = env['JINJALOADER']
        except KeyError:
            pass
        overlay = self.jinja_env.overlay(**options)
        template = overlay.get_template(str(source[0]))

        targetfile = open(str(target[0]), 'wb')
        targetfile.write(template.render(context).encode('utf-8'))
        targetfile.close()

    jinja_action.__name__ = 'Jinja'



# jinja-tools
#
# Various tools for Jinja2,
# including new filters and tests based on python-moretools,
# a JinjaLoader class for Django,
# and a simple JinjaBuilder class for SCons.
#
# Copyright (C) 2011-2013 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# jinja-tools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# jinja-tools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with jinja-tools.  If not, see <http://www.gnu.org/licenses/>.

from django.core import urlresolvers

import jinjatools

all = 'JinjaLoader',

class Template(object):
  def __init__(self, template):
    self.template = template

  def render(self, context):
    dicts = context.dicts
    context = {}
    for d in dicts:
      context.update(d)

    return self.template.render(context)

class Django(object):
    def url(self, name):
        return urlresolvers.reverse(name)

class LoaderFactory(object):
  def __getitem__(self, DjangoLoader):
    class JinjaLoader(DjangoLoader):
      def __init__(
        self, jinja_loader, filters = {}, tests = {}, globals = {},
        **kwargs
        ):
        DjangoLoader.__init__(self)

        globals = dict(globals)
        globals.update(
          django=Django(),
          )
        self.jinja_env = jinjatools.Environment(
          loader = jinja_loader,
          filters = filters, tests = tests, globals = globals)

      def load_template(self, name, dirs):
        if name.startswith('jinja:'):
          name = name.split('jinja:', 1)[1]
          source, origin = self.load_template_source(name, dirs)
          return (
            Template(self.jinja_env.from_string(source)),
            origin)

        return DjangoLoader.load_template(self, name, dirs)

    return JinjaLoader

JinjaLoader = LoaderFactory()



# jinja-tools
#
# various tools for Jinja2
# including new filters and tests based on python-moretools,
# a JinjaLoader class for Django
# and a simple JinjaBuilder class for SCons
#
# Copyright (C) 2011-2013 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# jinja-tools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# jinja-tools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with jinja-tools.  If not, see <http://www.gnu.org/licenses/>.

from moretools import *

__all__ = 'filters',

def dictremove(dict_, *keys):
  for key in keys:
    try: del dict_[key]
    except KeyError: pass

  return dict_

def dictupdate(dict_, items = (), **kwitems):
  dict_.update(items, **kwitems)
  return dict_

filters = dict(
  mapattr = mapattr,
  mapmapattr = mapmapattr,

  mapattrs = mapattrs,
  mapmapattrs = mapmapattrs,

  mapitem = mapitem,
  mapmapitem = mapmapitem,

  mapitems = mapitems,
  mapmapitems = mapmapitems,

  mapmethodcall = mapmethodcall,
  mapmapmethodcall = mapmapmethodcall,

  mapjoin = lambda seqs, sep: map(
    lambda seq: sep.join(map(str, seq)),
    seqs),

  zip = lambda seq, *seqs: zip(*chain(seq, seqs)),
  zipwith = zip,

  tee = tee,

  repeat = repeat,

  chain = lambda seq, *seqs: chain(*chain(seq, seqs)),
  chainwith = chain,

  combine = combinations,
  permutate = permutations,

  cross = lambda seq, *seqs: product(*chain(seq, seqs)),
  crosswith = product,

  dict = dict,

  dictremove = dictremove,
  dictupdate = dictupdate,

  partial = partial,

  split = str.split,
  )



# jinja-tools
#
# Various tools for Jinja2,
# including new filters and tests based on python-moretools,
# a JinjaLoader class for Django,
# and a simple JinjaBuilder class for SCons.
#
# Copyright (C) 2011-2013 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# jinja-tools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# jinja-tools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with jinja-tools.  If not, see <http://www.gnu.org/licenses/>.

from .filters import filters
from .env import Environment



from setuptools import setup

setup(
  name = 'jinjatools',
  version = '0.1a12',
  description = (
    'Various tools for Jinja2,'
    ' including new filters and tests based on moretools,'
    ' a JinjaLoader class for Django,'
    ' and a simple JinjaBuilder class for SCons.'
    ),
  author = 'Stefan Zimmermann',
  author_email = 'zimmermann.code@gmail.com',
  url = 'http://bitbucket.org/userzimmermann/python-jinjatools',

  license = 'LGPLv3',

  install_requires = [
    'six',
    'moretools >= 0.1a25',
    'Jinja2',
    ],
  packages = [
    'jinjatools',
    'jinjatools.django',
    'jinjatools.scons',
    ],

  classifiers = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved ::'
    ' GNU Library or Lesser General Public License (LGPL)',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Topic :: Software Development',
    'Topic :: Utilities',
    ],
  keywords = [
    'jinja', 'jinja2', 'tools', 'template', 'macro',
    'filter', 'test', 'moretools',
    'scons', 'builder', 'build',
    'django', 'web', 'html',
    'python3',
    ],
  )
